package com.lnb.multidatasourcedemo.listener;


import com.lnb.multidatasourcedemo.curator.CuratorClient;
import com.lnb.multidatasourcedemo.datasource.DynamicDataSource;
import com.lnb.multidatasourcedemo.model.DatasourceModel;
import com.lnb.multidatasourcedemo.utils.JsonUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.curator.framework.CuratorFramework;
import org.apache.curator.framework.recipes.cache.PathChildrenCacheEvent;
import org.apache.curator.framework.recipes.cache.PathChildrenCacheListener;

/**
 * 数据库配置监听
 * Created by liaoningbo on 2021/11/24
 */
@Slf4j
public class DbconfigListener implements PathChildrenCacheListener {

    private DynamicDataSource dynamicDataSource;

    private CuratorClient curatorClient;

    public DbconfigListener(DynamicDataSource dynamicDataSource, CuratorClient curatorClient) {
        this.dynamicDataSource = dynamicDataSource;
        this.curatorClient = curatorClient;
    }

    @Override
    public void childEvent(CuratorFramework client, PathChildrenCacheEvent event) throws Exception {
        log.info("DbconfigListener event type : [" + event.getType() + "] ");
        switch (event.getType()) {
            case CHILD_ADDED:
            case CHILD_UPDATED:
                registerDatasource(event);
                break;
            case CHILD_REMOVED:
                removeDatasource(event);
                break;
            default:
                break;
        }
    }

    private void registerDatasource(PathChildrenCacheEvent event) throws Exception {
        String dbconfig = curatorClient.read(event.getData().getPath());
        //添加节点时内容可能为空，这时注册节点下路径监听器
        if (PathChildrenCacheEvent.Type.CHILD_ADDED.equals(event.getType())) {
            curatorClient.registerRoutingKeyListenter(dynamicDataSource, getDB(event.getData().getPath()), getApp(event.getData().getPath()));
        }
        if (StringUtils.isBlank(dbconfig)) {
            return;
        }
        DatasourceModel datasourceModel = JsonUtil.json2object(dbconfig, DatasourceModel.class);
        dynamicDataSource.registerDataSource(datasourceModel);
    }

    private void removeDatasource(PathChildrenCacheEvent event) throws Exception {
        String path = event.getData().getPath();
        String key = getDB(path);
        dynamicDataSource.removeDatasource(key);
    }

    private String getDB(String path) {
        return path.substring(path.lastIndexOf("/") + 1, path.length());
    }

    private String getApp(String path) {
        String[] splits = path.split("/");
        if (splits.length >= 4)
            return splits[3];
        return "";
    }
}
