package com.lnb.multidatasourcedemo.listener;

import com.lnb.multidatasourcedemo.curator.CuratorClient;
import com.lnb.multidatasourcedemo.datasource.DynamicDataSource;
import lombok.extern.slf4j.Slf4j;
import org.apache.curator.framework.CuratorFramework;
import org.apache.curator.framework.recipes.cache.ChildData;
import org.apache.curator.framework.recipes.cache.PathChildrenCacheEvent;
import org.apache.curator.framework.recipes.cache.PathChildrenCacheListener;

import java.util.ArrayList;
import java.util.List;

/**
 * 路由监听.
 * Created by liaoningbo on 2021/11/24
 */
@Slf4j
public class RoutingKeyListener implements PathChildrenCacheListener {

    private DynamicDataSource dynamicDataSource;

    private CuratorClient curatorClient;

    public RoutingKeyListener(DynamicDataSource dynamicDataSource, CuratorClient curatorClient) {
        this.dynamicDataSource = dynamicDataSource;
        this.curatorClient = curatorClient;
    }

    @Override
    public void childEvent(CuratorFramework client, PathChildrenCacheEvent event) throws Exception {
        log.debug("RoutingKeyListener event type : [" + event.getType() + "] ");
        switch (event.getType()) {
            case INITIALIZED:
                List<ChildData> childDataList = event.getInitialData();
                if(childDataList == null || childDataList.isEmpty()){
                    break;
                }

                String path = childDataList.get(0).getPath();
                log.info("APP:{} DB:{} RoutingKey count:{} INITIALIZED finished", getApp(path), getDB(path), childDataList.size());
                break;
            case CHILD_ADDED:
            case CHILD_UPDATED:
                registerRouting(event);
                break;
            case CHILD_REMOVED:
                removeRouting(event);
                break;
            default:
                break;
        }
    }

    private void registerRouting(PathChildrenCacheEvent event) throws Exception {
        String path = event.getData().getPath();
        List<String> routingKeyList = new ArrayList<String>();
        routingKeyList.add(getRoutingKey(path));
        dynamicDataSource.registerRoutingKey(getDB(path), routingKeyList, getApp(path));
    }

    private void removeRouting(PathChildrenCacheEvent event) throws Exception {
        String path = event.getData().getPath();
        dynamicDataSource.removeRoutingKey(getRoutingKey(path));
        curatorClient.listenerList.remove(path);
    }


    private String getApp(String path) {
        String[] split = path.split("/");
        if (split.length >= 4) {
            return split[3];
        } else
            throw new IllegalStateException("App url is empty!");
    }

    private String getDB(String path) {
        String[] split = path.split("/");
        if (split.length >= 5) {
            return split[4];
        } else
            throw new IllegalStateException("DB url is empty!");
    }

    private String getRoutingKey(String path) {
        String[] split = path.split("/");
        if (split.length >= 5) {
            return split[5];
        } else
            throw new IllegalStateException("DB routingKey is empty!");
    }

}
